//
//	Battery discharge capacities meter 0.1
//		led7seg.c: 7 segment LED support
//		2009.04.23 1st rel
//			(c)2009 T.Togashi
//

#include "htc.h"
#define	bitset(var,bitno)	((var) |= 1UL << (bitno))
#define bitclr(var,bitno)	((var) &= ~(1UL << (bitno)))
#define bittest(var,bitno)	((var) & (1UL << (bitno)))
#define LED7SEG_DPBIT	1

static unsigned char LED7SEG_Loc = 0;
static unsigned char LED7SEG_Dat[4] = {0, 0, 0, 0};
const  unsigned char LED7SEG_Pat[16] = {	
//
//			 AAA
//			F   B
//			 GGG
//			E   C
//		 	 DDD *
//
//		  EABFGC*D
		0b11110101,		// 0
		0b00100100,		// 1
		0b11101001,		// 2
		0b01101101,		// 3
		0b00111100,		// 4
		0b01011101,		// 5
		0b11011101,		// 6
		0b01110100,		// 7
		0b11111101,		// 8
		0b01111101,		// 9
		0b11111100,		// A
		0b00011101,		// B
		0b10001001,		// C
		0b10101101,		// D
		0b11011001,		// E
		0b11011000		// F
};

void LED7SEG_PutPattern(unsigned char pat, unsigned char pos){
	unsigned char i;

	for(i = (pos==0 ? 1 : pos); i <= (pos==0 ? 3 : pos); i++) {
		unsigned char dp;

		dp = bittest(LED7SEG_Dat[i], LED7SEG_DPBIT);
		LED7SEG_Dat[i] = pat | dp;
	}
}
void LED7SEG_PutHex(unsigned char hex, unsigned char pos){
	unsigned char i;

	for(i = (pos==0 ? 1 : pos); i <= (pos==0 ? 3 : pos); i++) {
		unsigned char dp;

		dp = bittest(LED7SEG_Dat[i], LED7SEG_DPBIT);
		LED7SEG_Dat[i] = LED7SEG_Pat[hex] | dp;
	}
}
void LED7SEG_DP(unsigned char f, unsigned char pos){
	unsigned char i;

	for(i = (pos==0 ? 1 : pos); i <= (pos==0 ? 3 : pos); i++) {
		if (f) {
			bitset(LED7SEG_Dat[i], LED7SEG_DPBIT);
		} else {
			bitclr(LED7SEG_Dat[i], LED7SEG_DPBIT);
		}
	}
}
void LED7SEG_DispInt(int iValue, unsigned char bPos, unsigned char bLen) {
	int a;
	unsigned char s;
	signed char l;

	l = bLen;
	if (iValue < 0) {
		s = 1;
		a = -iValue;
	} else {
		s = 0;
		a = iValue;
	}

	for(--l ; (l >= 0) && (a != 0); --l) {
		unsigned char b;

		b = a % 10;
		a = a / 10;
		LED7SEG_PutHex(b, bPos + l);
	}

	if (l >= 0) {
		if (s) {
			LED7SEG_PutPattern(0x08, bPos + l);		// minus pattern
			--l;
		}
		for( ; l >= 0; --l) {
			LED7SEG_PutPattern(0x00, bPos + l);		// zero suppless
		}
	}

	return;
}
void LED7SEG_Refresh(){
	switch(LED7SEG_Loc){
	case 0:
		bitset(PORTA, 4);
		PORTB = LED7SEG_Dat[1];
		bitclr(PORTA, 2);
		break;
	case 1:
		bitset(PORTA, 2);
		PORTB = LED7SEG_Dat[2];
		bitclr(PORTA, 3);
		break;
	default:
		bitset(PORTA, 3);
		PORTB = LED7SEG_Dat[3];
		bitclr(PORTA, 4);
		break;
	}
	LED7SEG_Loc++;
	if (LED7SEG_Loc >= 3) {
		LED7SEG_Loc = 0;
	}
}
